from math import ceil
from random import choice
import sys

import generator.tasksets as g
import model.serialize as s
from model.tasks import TaskSystem
from util.stats import Histogram

import litmus
import topology

def tweak_cost(cost, period, deadline):
    return (ceil(cost), period, deadline)


dists = [
    'exp-10-10-100-shp',
    'uni-light-shp',
    'uni-medium-shp',
    'bimo-light-shp',
    'bimo-medium-shp',
    ]

def make_partition(num):
    while True:
        d = choice(dists)
        exp = g.DIST[d]
        ts = exp(1, num, tweak=tweak_cost)
        if len(ts) == num:
            return ts, d

def make_partitions(cpus, num):
    return [make_partition(num)[0] for _ in xrange(cpus)]

def assign_locks(part):
    CS_LEN = 0.01 # 10 microseconds... keep it short

    # hard-coded for Ludwig atm.
    for task in part:
        if task.cost <= 1:
            num = 3
        elif task.cost <= 2:
            num = 6
        elif task.cost <= 5:
            num = 12
        elif task.cost <=10:
            num = 24
        else:
            num = 48

        # num requests spead out over all resources (resid = 0 <=> all)
        task.resmodel.append((0, num, CS_LEN))


def make_taskset_file(host, num, idx, schedulers=None, with_locking=False):
    parts = make_partitions(topology.CPUS[host], num)

    if with_locking:
        for part in parts:
            assign_locks(part)

    # default: all schedulers
    if schedulers is None:
        schedulers = topology.SCHEDULERS[host]

    for sched in schedulers:
        tss = list(parts)
        if sched in litmus.RELEASE_MASTER:
            # drop first partition
            tss[0] = []
        if sched in litmus.GLOBAL:
            csize = len(tss)
        elif sched in litmus.PARTITIONED:
            csize = 1
        else:
            # clustered
            name, cache, rm = litmus.SCHEDULERS[sched]
            if cache == 'ALL': #emulate global
                csize = len(tss)
            else:
                csize = len(topology.HOSTS[host][cache][0])
        for i, ts in enumerate(tss):
            cluster = i / csize
            for t in ts:
                t.partition = cluster

        taskset = TaskSystem()
        for ts in tss:
            taskset.extend(ts)

        if with_locking:
            (plugin, x, y) = litmus.SCHEDULERS[sched]
            lock_protos    = litmus.LOCKING_PROTOCOLS[plugin]
            fnames = ['taskset_scheduler=%s_locks=%s_host=%s_n=%d_idx=%02d.ts' %
                      (sched, proto, host, num, idx) for proto in lock_protos]
        else:
            fname = 'taskset_scheduler=%s_host=%s_n=%d_idx=%02d.ts' % \
                (sched, host, num, idx)
            fnames = [fname]

        for fname in fnames:
            s.write_ts(taskset, fname)


def make_tasksets(host, max_idx, schedulers=None, with_locking=False, max_num=20):
    for num in xrange(1, max_num + 1):
        print '%2d tasks ' % num,
        sys.stdout.flush()
        for idx in xrange(1, max_idx + 1):
            make_taskset_file(host, num, idx, schedulers, with_locking)
            print '*',
            sys.stdout.flush()
        print

def show_dist(d, ucap, min=0, max=None, tweak=None):
    if max is None:
        max = ucap * 10
    exp = g.DIST[d]
    h   = Histogram(min, max, max - min)
    for _ in xrange(1000):
        ts = exp(ucap, None, tweak=tweak)
        h.count(len(ts))
    print "***", d, "*" * (75 - len(d))
    print h.as_ascii_bars()
    print

def show_dists():
    for d in dists:
        show_dist(d, 1, 0, 20,
                  tweak=tweak_cost)
