#!/usr/bin/env python
import sys

import model.serialize as s
import generator.tasksets as g

def generate_testpoints(util_range, wss_range,
                        util='exp-10-10-100',
                        deadline='implicit',
                        samples=100,
                        progress=True):

    udist = g.DIST[util]
    ddist = g.DEADLINE_DIST[deadline]
    generate = g.generate_with_deadlines(udist, ddist)

    total = len(util_range) * len(wss_range)
    cur = 0

    for u in util_range:
        for w in wss_range:
            cur += 1
            if progress:
                print "[%4d/%4d] %s/%s ucap=%.2f wss=%d" % \
                    (cur, total, util, deadline, u, w)
                sys.stdout.flush()

            tss = [generate(u, None) for _ in xrange(samples)]
            params = { 'dist'      : util,
                       'deadlines' : deadline,
                       'wss'       : w,
                       'ucap'      : u }
            ok_name = util.replace('_', '-')
            fname = "testpoint_ucap=%05.2f_wss=%04d_dist=%s_deadlines=%s.xml" % \
                (u, w, ok_name, deadline)
            s.write_testpoint(tss, params, fname)



def value_range(start, max, step):
    x  =  start
    while x <= max:
        yield x
        x += step


DISTS   = ['uni', 'bimo', 'exp']
WEIGHTS = ['light', 'medium', 'heavy']
PERIODS = [33, 100, 250]

EXP_LOOKUP = {
    'light'  : '-10',
    'medium' : '-25',
    'heavy'  : '-50',
    33       : '-3-33',
    100      : '-10-100',
    250      : '-50-250',
    }


def dist_name(dist, weight, period):
    if dist == 'exp':
        weight = EXP_LOOKUP[weight]
        period = EXP_LOOKUP[period]
        return "%s%s%s" % (dist, weight, period)
    else:
        return "%s_%s%s" % (dist, weight, "_%d" % period if period != 100 else "")

UDIST = []

for dist in DISTS:
    for weight in WEIGHTS:
        for per in PERIODS:
            UDIST.append(dist_name(dist, weight, per))

UCAP = list(value_range(1, 24.1, 0.25))
WSS  = range(0, 256, 16) + range(256, 1024, 64) + range(1024, 3073, 256)

def generate_files(dists=UDIST):
    for i, d in enumerate(dists):
        print "[%2d/%2d] Generating %s task sets..." % (i + 1, len(dists), d)
        generate_testpoints(UCAP, WSS, util=d)


def main(args=sys.argv):
    dists = args[1:]
    if not dists:
        print " ".join(UDIST)
    else:
        for d in dists:
            if not d in g.DIST:
                print "Unkown: %s"  % d
        generate_files([d for d in dists if d in g.DIST])


if __name__ == '__main__':
    main()

